import java.util.List;
import java.util.ArrayList;
import java.util.Collections;
/**
 * Write a description of class Game here.
 * 
 * @author (your name) 
 * @version (a version number or a date)
 */
public class Game
{
    private ArrayList<Card> myDeck;
    private ArrayList<Card> myPile;
    private ArrayList<Player> myPlayers;

    private int myCurrentPlayer;
    private int myPlayDirection;

    public Game(ArrayList<Card> deck, int nPlayers) {
        myDeck = deck;
        
        // Pile is initially empty;
        myPile = new ArrayList<Card>();
        myPlayers = new ArrayList<Player>();

        for (int i = 1; i <= nPlayers; i++) {
            Player p = new Player("Player " + i);
            myPlayers.add(p);
        }

    }

    public void reset() {
        myDeck.addAll(myPile);
        myPile.clear();
        Collections.shuffle(myDeck);
    }
    
    public Player getCurrentPlayer() {
        return myPlayers.get(myCurrentPlayer);
    }

    public Player getNextPlayer(int skip) {
        int i = myCurrentPlayer + (skip * myPlayDirection);
        i = i % myPlayers.size();
        if (i < 0) {
            i += myPlayers.size();
        }
        
        return myPlayers.get(i);
    }

    
    public int getPlayDirection() {
        return myPlayDirection;
    }
    
    public void setPlayDirection(int dir) {
        myPlayDirection = dir;
    }
    
    public void goToNextPlayer() {
        myCurrentPlayer = (myCurrentPlayer + myPlayDirection + myPlayers.size()) % myPlayers.size();
    }
    
    public void draw(Player player, int nCards) {
        System.out.println(player + " draws " + nCards + ".");

        for (int i = 0; i < nCards; i++) {
            if (myDeck.size() == 0) {
                Card keep = myPile.get(0);
                myDeck.addAll(myPile.subList(1, myPile.size() - 1));
                myPile.clear();
                myPile.add(keep);
                Collections.shuffle(myDeck);
            }
            
            if (myDeck.size() == 0) {
                // we've run out of cards, even when the pile is shuffled
                // just stop drawing
                return;
            }
            
            player.gainCard(myDeck.remove(0));
        }
    }
    
    public Player playTurn() {               
        Player player = getCurrentPlayer();
        
        for (Player p : myPlayers) {
            if (p == player) {
                System.out.print("* ");
            }
            System.out.println(p + ": " + p.getCards());
        }
        
        Card card = player.playCard(myPile.get(0));
        
        if (card == null) {
            // draw
            System.out.println(player + " cannot play.");
            draw(player, 1);
        }
        else {
            System.out.println(player + " plays " + card + ".");
            myPile.add(0, card);
            card.play(this);
        }
        
        if (player.getCards().size() == 0) {
            return player;
        }
        
        System.out.println();
        goToNextPlayer();
        return null;
    }
    
    public void deal() {
        for (Player p : myPlayers) {
            draw(p, 5);
        }
        
        // turn over the top card
        Card card = myDeck.remove(0);
        myPile.add(0, card);
        System.out.println("The starting card is: " + card);

        myCurrentPlayer = 0;
        myPlayDirection = 1;
    }
    
    public Player play() {
    
        deal();
        Player winner = null;
        
        while (winner == null) {
            winner = playTurn();
        }
        
        System.out.println(winner + " has won the game!");
        return winner;
    }
    
    public static List<Card> makeStandardDeck() {
        List<Card> deck = new ArrayList<Card>();
        
        // four of each number/color combination
        for (int i = 0; i < 4; i++) {
            for (int n = 0; n <= 9; n++) {
                for (int c = 1; c <= Card.NUM_COLOURS; c++) {
                    deck.add(new NumberCard(n, c)); 
                }
            }
        }
        
        // two of each action card
        for (int i = 0; i < 2; i++) {
            for (int c = 1; c <= Card.NUM_COLOURS; c++) {
                deck.add(new SkipCard(c)); 
                deck.add(new DrawCard(c, 2)); 
                deck.add(new ReverseCard(c)); 
            }
        }
        
        // four of each of the wild cards
        for (int i = 0; i < 4; i++) {
            deck.add(new WildCard());
            deck.add(new WildDrawCard(4));
        }
        
        // shuffle
        Collections.shuffle(deck);
        return deck;
    }
}
